@@ -110,7 +110,6 @@
                                         "/path/to/project/compiled"))))
         (should (equal (projectile-project-ignored) files))))))
 
-
 (ert-deftest projectile-remove-ignored-suffixes ()
   (noflet ((projectile-project-root () "/path/to/project")
            (projectile-project-name () "project")
@@ -123,6 +122,41 @@
                              (mapcar 'projectile-expand-root
                                      '("foo.c" "foo.o.gz"))))))))
 
+(ert-deftest projectile-add-unignored-files ()
+  (noflet ((projectile-get-repo-ignored-files () '("unignored-file"
+                                                   "path/unignored-file2")))
+    (let ((projectile-globally-unignored-files '("unignored-file")))
+      (should (equal (projectile-add-unignored '("file"))
+                     '("file" "unignored-file"))))
+    ;; Files inside ignored paths need to be explicitely unignored
+    (let ((projectile-globally-unignored-files '("unignored-file"
+                                               "path/unignored-file2")))
+      (should (equal (projectile-add-unignored '("file"))
+                     '("file" "unignored-file" "path/unignored-file2"))))))
+
+(ert-deftest projectile-add-unignored-files-no-vcs ()
+  (noflet ((projectile-project-vcs () 'none))
+    ;; Fail when no VCS command for ignored files is found
+    (let ((projectile-globally-unignored-files '("unignored-file")))
+      (should-error (projectile-add-unignored '("file"))))
+    ;; But only fail if unignored files are requested
+    (let (projectile-globally-unignored-files)
+      (should (equal (projectile-add-unignored '("file")) '("file"))))))
+
+(ert-deftest projectile-add-unignored-directories ()
+  (noflet ((projectile-project-vcs () 'git)
+           (projectile-get-repo-ignored-files () '("path/unignored-file")))
+    (let ((projectile-globally-unignored-directories '("path")))
+      (should (equal (projectile-add-unignored '("file"))
+                     '("file" "path/unignored-file")))
+      ;; Ignored files inside unignored paths need to be explicitely
+      ;; unignored
+      (let ((projectile-globally-ignored-files '("unignored-file")))
+        (should (equal (projectile-add-unignored '("file"))
+                       '("file")))
+        (let ((projectile-globally-unignored-files '("path/unignored-file")))
+          (should (equal (projectile-add-unignored '("file"))
+                       '("file" "path/unignored-file"))))))))
 
 (ert-deftest projectile-test-parse-dirconfig-file ()
   (noflet ((file-exists-p (filename) t)
@@ -132,8 +166,9 @@
             (save-excursion
               (insert
                "\n-exclude\n+include\nno-prefix\n left-wspace\nright-wspace\t\n"))))
-    (should (equal '(("include/") .
-                     ("exclude" "no-prefix" "left-wspace" "right-wspace"))
+    (should (equal '(("include/")
+                     ("exclude" "no-prefix" "left-wspace" "right-wspace")
+                     nil)
                    (projectile-parse-dirconfig-file)))))
 
 (ert-deftest projectile-test-get-project-directories ()
@@ -151,13 +186,14 @@
 (ert-deftest projectile-test-dir-files ()
   (noflet ((projectile-project-root () "/my/root/")
            (projectile-patterns-to-ignore () nil)
-           (projectile-index-directory (dir patterns) (should (equal dir "a/"))
+           (projectile-index-directory (dir patterns progress-reporter)
+                                       (should (equal dir "a/"))
                                        '("/my/root/a/b/c" "/my/root/a/d/e"))
            (projectile-get-repo-files () '("/my/root/a/b/c" "/my/root/a/d/e"))
            (cd (directory) "/my/root/a/" nil))
-    (let ((projectile-use-native-indexing t))
+    (let ((projectile-indexing-method 'native))
       (should (equal '("a/b/c" "a/d/e") (projectile-dir-files "a/"))))
-    (let ((projectile-use-native-indexing nil))
+    (let ((projectile-indexing-method 'alien))
       (should (equal '("a/b/c" "a/d/e") (projectile-dir-files "a/"))))))
 
 (ert-deftest projectile-test-setup-hook-functions-projectile-mode ()
@@ -182,7 +218,8 @@
       (should (equal (projectile-relevant-known-projects) '("/path/to/project2"))))))
 
 (ert-deftest projectile-test-projects-cleaned ()
-  (let* ((directories (cl-loop repeat 3 collect (make-temp-file "projectile-cleanup" t)))
+  (let* ((projectile-known-projects-file (projectile-test-tmp-file-path))
+         (directories (cl-loop repeat 3 collect (make-temp-file "projectile-cleanup" t)))
          (projectile-known-projects directories))
     (unwind-protect
         (progn
@@ -191,7 +228,8 @@
           (delete-directory (car directories))
           (projectile-cleanup-known-projects)
           (should (equal projectile-known-projects (cdr directories))))
-      (--each directories (ignore-errors (delete-directory it))))))
+      (--each directories (ignore-errors (delete-directory it)))
+      (delete-file projectile-known-projects-file nil))))
 
 (ert-deftest projectile-test-project-root-is-absolute ()
   (let* ((root-directory (make-temp-file "projectile-absolute" t))
@@ -453,6 +491,23 @@
               (should (member f (gethash (projectile-project-root)
                                          projectile-projects-cache))))))))))
 
+(ert-deftest projectile-test-old-project-root-gone ()
+  "Ensure that we don't cache a project root if the path has changed."
+  (projectile-test-with-sandbox
+    (projectile-test-with-files
+        ("project/"
+         "project/.projectile")
+      (cd "project")
+      (let* ((projectile-project-root-cache (make-hash-table :test #'equal))
+             (correct-project-root (projectile-project-root)))
+        ;; If this project has been moved, then we will have stale
+        ;; paths in the cache.
+        (puthash
+         (format "projectile-root-bottom-up-%s" correct-project-root)
+         "/this/path/does/not/exist"
+         projectile-project-root-cache)
+        (should (string= (projectile-project-root) correct-project-root))))))
+
 (ert-deftest projectile-test-git-grep-prefix ()
   (require 'vc-git)
   (projectile-test-with-sandbox
@@ -596,7 +651,10 @@
                                                     "src/food/cat.c")))
   (should (equal 0
                  (projectile-dirname-matching-count "src/weed/sea.c"
-                                                    "src/food/sea.c"))))
+                                                    "src/food/sea.c")))
+  (should (equal 0
+                 (projectile-dirname-matching-count "test/demo-test.el"
+                                                    "demo.el"))))
 
 (ert-deftest projectile-test-find-matching-test ()
   (projectile-test-with-sandbox
@@ -636,6 +694,31 @@
                          (projectile-find-matching-file
                           "spec/models/food/sea_spec.rb"))))))))
 
+(ert-deftest projectile-test-exclude-out-of-project-submodules ()
+  (projectile-test-with-files
+      (;; VSC root is here
+       "project/"
+       "project/.git/"
+       "project/.gitmodules"
+       ;; Current project root is here:
+       "project/web-ui/"
+       "project/web-ui/.projectile"
+       ;; VCS git submodule will return the following submodules,
+       ;; relative to current project root, 'project/web-ui/':
+       "project/web-ui/vendor/client-submodule/"
+       "project/server/vendor/server-submodule/")
+    (let ((project (file-truename (expand-file-name "project/web-ui"))))
+      (noflet ((projectile-files-via-ext-command
+                (arg) (when (string= default-directory project)
+                        '("vendor/client-submodule"
+                          "../server/vendor/server-submodule")))
+               (projectile-project-root
+                () project))
+
+        ;; assert that it only returns the submodule 'project/web-ui/vendor/client-submodule/'
+        (should (equal (list (expand-file-name "vendor/client-submodule/" project))
+                       (projectile-get-all-sub-projects project)))))))
+
 ;; Local Variables:
 ;; indent-tabs-mode: nil
 ;; End:
